/* Copyright © 2013 Matthew Champion
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of mattunderscore.com nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MATTHEW CHAMPION BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

package com.mattunderscore.http.headers;

import java.util.ArrayList;
import java.util.Collection;

/**
 * Parse an accept encoding field header string to a structured representation.
 * 
 * A special note is required for the identity encoding. The identity encoding is always acceptable
 * unless it explicitly or by wild card has a qualifier of 0. Where the identity is acceptable it
 * shall have the quality value a tenth less than the smallest other quality value. The quality
 * value is not part of the specification.
 * 
 * @author Matt Champion
 * @since 0.1.2
 */
public class EncodingParser implements HeaderParser<QEncoding>
{

    public EncodingParser()
    {
    }

    @Override
    public boolean isCorrectHeader(String header)
    {
        return "Accept-Encoding".equalsIgnoreCase(header);
    }

    @Override
    public Collection<QEncoding> parseAll(String header) throws UnParsableHeaderException
    {
        Collection<QEncoding> qEcodings = new ArrayList<QEncoding>();
        String[] types = header.split(",");
        boolean addIdentity = true;
        double smallestQualifier = 10.0;
        for (int i = 0; i < types.length; i++)
        {
            QEncoding encoding = parse(types[i].trim());
            if (encoding != null)
            {
                if (encoding.sameEncoding(new QEncoding("identity", 1.0)))
                {
                    addIdentity = false;
                }
                else if (encoding.isAnyEncoding())
                {
                    addIdentity = false;
                }
                else if (addIdentity && encoding.getQualifier() < smallestQualifier)
                {
                    smallestQualifier = encoding.getQualifier();
                }
                qEcodings.add(encoding);
            }
        }
        if (addIdentity)
        {
            qEcodings.add(new QEncoding("identity", smallestQualifier / 10.0));
        }
        return qEcodings;
    }

    @Override
    public QEncoding parse(String header) throws UnParsableHeaderException
    {
        if (header == null)
        {
            throw new UnParsableHeaderException("Null Pointer.");
        }
        else if ("".equals(header.trim()))
        {
            return null;
        }
        else
        {
            String[] headers = header.split(";q=");
            if (headers.length == 2)
            {
                double qualifier = Double.parseDouble(headers[1].trim());
                if (qualifier < 0.0 || qualifier > 1.0)
                {
                    throw new UnParsableHeaderException("Invalid qualifier range.");
                }
                else
                {
                    return new QEncoding(headers[0].trim(), qualifier);
                }
            }
            else
            {
                return new QEncoding(headers[0].trim(), 1.0);
            }
        }
    }
}
